/* si4735.h - data type definitions etc. 

   Software for controlling the Si4735 based "PC radio
   with USB,  version 2 with 32 kHz xtal" from 
   AK MODL-BUS Computer GmbH 
   
   More info: http://www.elexs.de/SI4735i.html 

   This program is distributed under the GPL, version 2

   Copyright (C) 2011 Tobias Stoeber
   Based on demo program / Delphi code (C) Burkhard Kainka
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* ===============================
     misc things ....
   =============================== */

/* bitmask array */
const int Bitmask[8] = { 0, 1, 2, 4, 8, 16, 32, 64 };

/* data type for Si IC info */
struct si_chip_info
{
   char ic[5];
   char fw[3];
   char comp_fm[3];
   char comp_am[3];
   char chip_rev[3];  
};

/* debug flags */
struct dbg_flags
{
   int GPIO,
       RDS,
       RDSgroup,
       RTplus,
       ScreenCap,
       TMC;
};

/* ===============================
     LW/AM/SW frequency bands
   =============================== */

/* data type for freq band */
struct radio_band 
{
   int wavelength;	/* wavelenth band in m */
   int band_bottom;	/* bottom band border in kHz */
   int band_start_freq; /* start frequeny, differs in LW + MW */
   int band_top;	/* upper (top) band borde in kHz */
   int freq_spacing;	/* frequency spacing */
};

struct radio_band AM_bands [15] = {  
                                    { 1500, 150, 153, 520, 3 },  /* LW: 153 - 515 in (9) 3 kHz */
                                    {  300, 520, 522, 1710, 9 }, /* MW: (520) 522 - 1710 in 9 kHz */
                                    {  120, 2300, 2300, 2495, 5 },
                                    {   90, 3200, 3200, 3400, 5 },
                                    {   75, 3900, 3900, 4000, 5 },
                                    {   60, 4750, 4750, 5060, 5 },
                                    {   49, 5950, 5950, 6200, 5 },
                                    {   41, 7100, 7100, 7350, 5 },
                                    {   31, 9400, 9400, 9900, 5 },
                                    {   25, 11600, 11600, 12100, 5 },
                                    {   22, 13570, 13570, 13870, 5 },
                                    {   19, 15100, 15100, 15800, 5 },
                                    {   16, 17480, 17480, 17900, 5 },
                                    {   13, 21450, 21450, 21750, 5 },
                                    {   11, 25600, 25600, 26100, 5 },
                                 };

/* ===============================
     Presets
   =============================== */


/* data type for preset information */
struct preset 
{
   int freq;
   char station_name[32];
   int rds_pi;
}; 

/* ===============================
     RDS: RadioText Plus
   =============================== */

/* data type for RDS RadioText Plus tags */
struct rt_plus_tag
{
   int type;
   int start;
   int length;
   int toggle;
   int running;
   int ab_flag;
   int cnt;
   char *str;
};

/* data type for RDS RadioText Plus Tag Types */
struct rt_plus_code
{
   char rtp_class[32];
   char desc[256]; 
};

struct rt_plus_code RTplus_Codes[64] = {
   {	"DUMMY_CLASS", 		"class to assign, if RadioText contains no RT+ information" },
   {	"ITEM.TITLE",		"title of item, e.g. track title of album" },
   {	"ITEM.ALBUM",		"the collection name to which the track belongs" },
   {	"ITEM.TRACKNUMBER",	"the track number of the item on the album on which it was originally released" },
   {	"ITEM.ARTIST",		"a person or band/collective generally considered to be responsible for the work" },
   {	"ITEM.COMPOSITION",	"a complete composition" },
   {	"ITEM.MOVEMENT",	"a movement is a large division of a composition or musical form" },
   { 	"ITEM.CONDUCTOR",	"the artist(s) who performed the work" },
   {	"ITEM.COMPOSER",	"name of the original composer / author" },
   {	"ITEM.BAND",		"band/orchestra/accompaniment/musician" },
   {	"ITEM.COMMENT", 	"any comment related to the content" },
   {	"ITEM.GENRE",		"the main genre of the audio, e.g. classical, hip-hop, jazz, oldies, drama etc." },
};

/* ===============================
     Si4735 FM and AM properties
   =============================== */

/* data typ and data for receiver property defs */
struct si4735_property 
{
   int prop;		/* property */
   char name[32];	/* property name as defined in AN332 */
   char name2[32];	/* property name as defined in AN332 */
   char desc[256];	/* property description as defined in AN332 */
   int val_def;		/* property's default value as defined in AN332 */
   int val_curr;	/* current value */
   int val_old; 	/* old value */
   int val_min;		/* minimum value */
   int val_max; 	/* maximum value */
   int val_step;	/* step in value range */
   char units[32];	/* unit description */
   int isDiscrete;	/* discrete values (1) vs. range (0) */ 
   char *val_disc[32];
};                                

struct si4735_property	SI4735_prop_fm[6] = {
   { 	0x1100, 	"FM_DEEMPHASIS",
                        "FM Deemphasis",
                        "Sets deemphasis time constant. Default is 75 us",
                        0x0002, -1, 0,
                        1, 2, 1,
                        "",
                        1,
                        { "-n/a-", "50 us", "75us" },
                     }, /* 1 = 50 us, 2 = 75 us */

   { 	0x1102, 	"FM_CHANNEL_FILTER",
                        "FM Channel Filter",
                        "Selects bandwidth of channel filter applied at the demodulation stage. Default is automatic which means the device automatically selects proper channel filter.",
                        0x0001, -1, 0,
                        0, 4, 1,
                        "",
                        1,
                        { "auto", "110 kHz wide", " 84 kHz narrow", " 60 kHz narrower", " 40 kHz narrowest"}, 
                     },

   {	0x1105,		"FM_BLEND_STEREO_THRESHOLD",
                        "FM Blend Stereo Threshold",
                        "Sets RSSI threshold for stereo blend (Full stereo above threshold, blend below threshold). To force stereo, set this to 0",
                        0x0031, -1, 0,
                        0, 127, 1, 
                        "dBµV",
                        0 },

   {	0x1106,		"FM_BLEND_MONO_THRESHOLD",
                        "FM Blend Mono Threshold",
                        "Sets RSSI threshold for mono blend (Full mono below threshold, blend above threshold). To force stereo, set this to 0. To force mono, set this to 127.",
                        0x001E, -1, 0,
                        0, 127, 1, 
                        "dBµV",
                        0 },

   {	0x1403, 	"FM_SEEK_SNR_THRESHOLD",
                        "FM Seek SNR Threshold",
                        "Sets the SNR threshold for a valid FM Seek/Tune. If the value is zero then SNR threshold is not considered when doing a seek. Default value is 3 dB.",
                        0x0003, -1, 0,
                        0, 127, 1, 
                        "dB",
                        0 },   
   
   {	0x1404, 	"FM_SEEK_RSSI_THRESHOLD",
                        "FM Seek RSSI Threshold",
                        "Sets the RSSI threshold for a valid FM Seek/Tune. If the value is zero then RSSI threshold is not considered when doing a seek. Default value is 20 dB.",
                        0x0014, -1, 0,
                        0, 127, 1, 
                        "dBµV",
                        0 },

/*
   { 	0x1502, 	"RDS_CONFIG",
                        "RDS Block Error Threshold",
                        "Configures RDS settings to enable RDS processing (RDSEN) and set RDS block error thresholds. When a RDS Group is received, all block errors must be less than or equal the associated block error threshold for the group to be stored in the RDS FIFO.",
                        0x0001, -1, 0,
                        1, 4, 1,
                        "",
                        1,
                        { "no errors", "1-2 bit err", "2-3 bit err", "uncorr. err"}, 
                     },
*/

};

struct si4735_property	SI4735_prop_am[7] = {
   { 	0x3102, 	"AM_CHANNEL_FILTER",
                        "AM Channel Filter",
                        "Selects the bandwidth of the AM channel filter. The 1 kHz option, 1.8 kHz option, 2.5 kHz option and 100 Hz high-pass Line Noise Rejection filter are supported on Si473x-C40 and later devices",
                        0x0003, -1, 0,
                        0, 6, 1,
                        "",
                        1,
                        { "6 KHz", "4 kHz", "3 kHz", "2 kHz", "1 kHz", "1.8 kHz", "2.5 kHz" }, 
                     },

   { 	0x3300, 	"AM_SOFT_MUTE_RATE",
                        "AM Soft Mute Rate",
                        "Sets the attack and decay rates when entering or leaving soft mute. The default is 278 dB/s.",
                        0x0040, -1, 0,
                        1, 255, 1,
                        "dB/s",
                        0 },
                        
   {	0x3301,		"AM_SOFT_MUTE_SLOPE",
                        "AM Soft Mute Slope",
                        "Sets the AM soft mute slope. Default value is a slope of 1.",
                        0x0001, -1, 0,
                        1, 5, 1,
                        "dB/dB",
                        0 },
                        
   {	0x3302,		"AM_SOFT_MUTE_MAX_ATTENUATION",
                        "AM Soft Mute Max Attenuation",
                        "Sets maximum attenuation during soft mute (dB). Set to 0 to disable soft mute. Default is 8 dB.",
                        0x0008, -1, 0,
                        0, 63, 1, 
                        "dB",
                        0 },
                        
   {	0x3303,		"AM_SOFT_MUTE_SNR_THRESHOLD",
                        "AM Soft Mute SNR Threshold",
                        "Sets SNR threshold to engage soft mute. Default is 8 dB.",
                        0x0008, -1, 0,
                        0, 63, 1, 
                        "dB",
                        0 },

   {	0x3403, 	"AM_SEEK_SNR_THRESHOLD",
                        "AM Seek SNR Threshold",
                        "Sets the SNR threshold for a valid AM Seek/Tune. If the value is zero then SNR threshold is not considered when doing a seek. Default value is 5 dB.",
                        0x0005, -1, 0,
                        0, 63, 1, 
                        "dB",
                        0 },   
   
   {	0x3404, 	"AM_SEEK_RSSI_THRESHOLD",
                        "AM Seek RSSI Threshold",
                        "Sets the RSSI threshold for a valid AM Seek/Tune. If the value is zero then RSSI threshold is not considered when doing a seek. Default value is 25 dB.",
                        0x0019, -1, 0,
                        0, 63, 1, 
                        "dBµV",
                        0 },
}; 
